@@ -1,7 +1,7 @@
 /**********************************************************************
  *  batchprocessing.h
  **********************************************************************
- * Copyright (C) 2020 MX Authors
+ * Copyright (C) 2020-2024 MX Authors
  *
  * Authors: Adrian
  *          MX Linux <http://mxlinux.org>
@@ -21,9 +21,7 @@
  * You should have received a copy of the GNU General Public License
  * along with MX Snapshot.  If not, see <http://www.gnu.org/licenses/>.
  **********************************************************************/
-
-#ifndef BATCHPROCESSING_H
-#define BATCHPROCESSING_H
+#pragma once
 
 #include <QTimer>
 
@@ -34,7 +32,7 @@ class Batchprocessing : public QObject, public Settings
 {
     Q_OBJECT
 public:
-    Batchprocessing(const QCommandLineParser &arg_parser, QObject *parent = nullptr);
+    explicit Batchprocessing(const QCommandLineParser &arg_parser, QObject *parent = nullptr);
 
     void setConnections();
 
@@ -44,6 +42,6 @@ public slots:
 private:
     Work work;
     QTimer timer;
-};
 
-#endif // BATCHPROCESSING_H
+    void checkNvidiaGraphicsCard();
+};
