@@ -1,7 +1,7 @@
 /**********************************************************************
  *  work.h
  **********************************************************************
- * Copyright (C) 2020 MX Authors
+ * Copyright (C) 2020-2024 MX Authors
  *
  * Authors: Adrian
  *          MX Linux <http://mxlinux.org>
@@ -21,9 +21,7 @@
  * You should have received a copy of the GNU General Public License
  * along with MX Snapshot.  If not, see <http://www.gnu.org/licenses/>.
  **********************************************************************/
-
-#ifndef WORK_H
-#define WORK_H
+#pragma once
 
 #include <QElapsedTimer>
 
@@ -39,21 +37,18 @@ public:
     enum HashType { md5, sha512 };
     Q_ENUM(HashType)
 
-    Work(Settings *settings, QObject *parent = nullptr);
+    explicit Work(Settings *settings, QObject *parent = nullptr);
+    friend class Settings;
+    friend class Batchprocessing;
+    friend class MainWindow;
 
-    QElapsedTimer e_timer;
-    bool done = false;
-    bool started = false;
-
-    QTemporaryDir initrd_dir;
+    [[nodiscard]] bool checkAndMoveWorkDir(const QString &dir, quint64 req_size);
+    [[nodiscard]] quint64 getRequiredSpace();
+    [[nodiscard]] static bool checkInstalled(const QString &package);
     [[noreturn]] void cleanUp();
-    bool checkAndMoveWorkDir(const QString &dir, quint64 req_size);
-    bool checkInstalled(const QString &package);
     bool createIso(const QString &filename);
     bool installPackage(const QString &package);
     bool replaceStringInFile(const QString &old_text, const QString &new_text, const QString &file_path);
-    quint64 getRequiredSpace();
-    static void writeSnapshotInfo();
     void checkEnoughSpace();
     void checkNoSpaceAndExit(quint64 needed_space, quint64 free_space, const QString &dir);
     void closeInitrd(const QString &initrd_dir, const QString &file);
@@ -65,6 +60,7 @@ public:
     void savePackageList(const QString &file_name);
     void setupEnv();
     void writeLsbRelease();
+    void writeSnapshotInfo();
     void writeUnsquashfsSize(const QString &text);
     void writeVersionFile();
 
@@ -74,6 +70,10 @@ signals:
 
 private:
     Settings *settings;
+    QString elevate {QFile::exists("/usr/bin/pkexec") ? "/usr/bin/pkexec" : "/usr/bin/gksu"};
+    Cmd shell;
+    QElapsedTimer e_timer;
+    QTemporaryDir initrd_dir;
+    bool done = false;
+    bool started = false;
 };
-
-#endif // WORK_H
